//+------------------------------------------------------------------+
//|                                sqConsecutiveCandlesStatistics.mq4
//+------------------------------------------------------------------+

#property indicator_chart_window

//---- indicator parameters
extern int BarsBack=0; // 0 = use all history
extern string objPrefix = "sqConsecutiveCandlesStatistics";
extern color  TextColor = White;//DimGray;

//---- indicator buffers

//---- vars

int consecutive[2]={0,0};
int consecutive_candles[10][2]; // consecutive_candles[n][dir=UP,DN]
//---magft---
double reversalCandleBodySize[100][100][2]; //store size of reversal candle for processing
double AvSize[100][2];
double StdDev[100][2];
double Profit[100][2];
int numReversals,numSamples;
double avTmp,sdTmp;
double profit;
//-----------
datetime dt;

double pip;
int pipMult,pipMultTab[]={1,10,1,10,1,10,100}; // multiplier to convert pips to Points;

//+------------------------------------------------------------------+
int init()
//+------------------------------------------------------------------+
{
  pipMult = pipMultTab[Digits];
  pip = Point * pipMult;

  RemoveObjects(objPrefix);
  dt = 0;
  start();
  return(0);
}

//---magft---
//+------------------------------------------------------------------+
int deinit()
//+------------------------------------------------------------------+
{
  RemoveObjects(objPrefix);
  return(0);
}
//----------

//+------------------------------------------------------------------+
int start()
//+------------------------------------------------------------------+
{
  int limit,i,j,dir,prev_dir,n=1,n_max=0;
//---magft---
  int k=0;
//-----------
  datetime time_max;
  double percent;
//  int counted_bars=IndicatorCounted();
  
#define UP 0
#define DN 1

  if(dt == Time[0]) return(0); // update at each bar open only
  dt = Time[0];

  limit=Bars;
  if (BarsBack>0) limit = MathMin(limit,BarsBack);

  for(i=0; i<=ArrayRange(consecutive_candles,0); i++) {
    consecutive_candles[i][UP] = 0;
    consecutive_candles[i][DN] = 0;
    AvSize[i][dir] = 0;
    StdDev[i][dir] = 0;
  }

  for(i=0; i<=ArrayRange(reversalCandleBodySize,0); i++) {
    for(j=0; j<=ArrayRange(reversalCandleBodySize,1); j++) {
      reversalCandleBodySize[i][j][UP] = 0;
      reversalCandleBodySize[i][j][DN] = 0;
    }
  }

  profit = 0;

  prev_dir = UP;
  for(i=limit-1; i>=0; i--)
  {
    dir = prev_dir;
    if (Close[i] > Open[i]) dir = UP;
    if (Close[i] < Open[i]) dir = DN;

    double bodySize = (MathAbs(Close[i]-Open[i])/pip);//calculates the candle body size and stores for processing

    // grow stat arrays if needed
    if (n+1 > ArrayRange(consecutive_candles,0)) {
      ArrayResize(consecutive_candles,n+1);
      ArrayResize(AvSize,n+1);
      ArrayResize(StdDev,n+1);
      ArrayResize(Profit,n+1);
    }

    if (dir == prev_dir) { // same direction increment count
      n++;
      if (n>n_max) {
        n_max    = n;
        time_max = Time[i];
      }
      Profit[n-1][dir] -= bodySize; // count in loss for the tarde that would have expected a reversal instead of a continuation

    } else { // direction changed: update stats and reset counters
      consecutive_candles[n][prev_dir] ++;
      consecutive_candles[0][prev_dir] ++; // [0] counts the total UP/DN sequences
      //---magft---
      AvSize[n][prev_dir] += bodySize; // cumulate bodies to calc the Avg later
      Profit[n][prev_dir] += bodySize; // count profit based on trading the full reversal candle
      numReversals = consecutive_candles[n][prev_dir];
      if (numReversals+1>ArrayRange(reversalCandleBodySize,0)) ArrayResize(reversalCandleBodySize,numReversals+1);
      reversalCandleBodySize[numReversals][n][prev_dir] = bodySize; // store BodySize value
      //if (n==4 && prev_dir==DN) Print(DoubleToStr(bodySize,0));
      //-----------
      n = 1; // we now have 1 bar in the new direction
      prev_dir = dir;
    }
  }
  
  //---magft---
  //Calculate the average candle body size and the StdDev
  for (dir=UP; dir<=DN; dir++) {
     for (n=1; n<ArrayRange(consecutive_candles,0); n++) {
        numReversals = consecutive_candles[n][dir];
        if (numReversals == 0) {
          AvSize[n][dir] = 0;
          StdDev[n][dir] = 0;
          continue; // no reversal for this n,dir
        }
        AvSize[n][dir] /= numReversals;
        for(k=1;k<numReversals;k++) {
           avTmp=avTmp+reversalCandleBodySize[k][n][dir];
        }
        AvSize[n][dir]=NormalizeDouble(avTmp/numReversals,Digits);
        avTmp=AvSize[n][dir];
        
        sdTmp=0;
        for(k=1;k<numReversals;k++) {
          sdTmp=sdTmp+MathPow((reversalCandleBodySize[k][n][dir]-avTmp),2);
        }
        sdTmp = MathSqrt(sdTmp/numReversals);
        StdDev[n][dir]=NormalizeDouble(sdTmp,Digits);
     }
  }
  //-----------

  // display stats results

  drawFixedLbl(objPrefix+".titleUp", "UP sequences:"+DoubleToStr(consecutive_candles[0][UP],0),   0, 10, 15, 10, "Courier", TextColor, true);
  drawFixedLbl(objPrefix+".titleDn", "DOWN sequences:"+DoubleToStr(consecutive_candles[0][DN],0), 0,360, 15, 10, "Courier", TextColor, true);
  plot (UP, -1, "n", "nb", "%", "prob", "Body", "SD", "Profit");
  plot (DN, -1, "n", "nb", "%", "prob", "Body", "SD", "Profit");
  plot (UP, 0, "---", "---", "---", "---", "---", "---", "---");
  plot (DN, 0, "---", "---", "---", "---", "---", "---", "---");

  for (dir=UP; dir<=DN; dir++) {
    double cumul=0, prob=0;
    for (n=1; n<ArrayRange(consecutive_candles,0); n++) {
      percent = consecutive_candles[n][dir]*100;
      percent /= consecutive_candles[0][dir];
      if (cumul!=100) prob = 100* percent / (100-cumul); else prob=0;
      cumul += percent;
      plot (dir, n, DoubleToStr(n,0), DoubleToStr(consecutive_candles[n][dir],0)
            ,DoubleToStr(percent,1)+"%", DoubleToStr(prob,1)+"%", DoubleToStr(AvSize[n][dir],1), DoubleToStr(StdDev[n][dir],1)
            ,DoubleToStr(Profit[n][dir],0));
    }
  }

  Print("Longest consecutive bars sequence = "+n_max+" bars at "+TimeToStr(time_max,TIME_DATE|TIME_MINUTES));
//---- done

  return(0);
}

string dirs[] = {"UP","DN"};

//+------------------------------------------------------------------+
void plot (int dir, int line, string n, string nb, string percent, string prob, string Av, string StdDev, string profit)
//+------------------------------------------------------------------+
{
  string prefix = "";
  int x,y;
  
  y=39+12*line;
  x=10+310*dir;
  if (dir==UP) drawFixedLbl(objPrefix+dir+".n"+n, n+":", 0, x,y, 10, "Courier", TextColor, true);
  x+=40; drawFixedLbl(objPrefix+dir+".nb"+n, nb, 0, x,y, 10, "Courier", TextColor, true);
  x+=40; drawFixedLbl(objPrefix+dir+".%"+n,percent, 0, x,y, 10, "Courier", TextColor, true);
  x+=50; drawFixedLbl(objPrefix+dir+".prob"+n,prob, 0, x,y, 10, "Courier", TextColor, true);
//  x+=50; drawFixedLbl(objPrefix+dir+".avg"+n,Av, 0, x,y, 10, "Courier", TextColor, true);
//  x+=50; drawFixedLbl(objPrefix+dir+".sd"+n,StdDev, 0, x,y, 10, "Courier", TextColor, true);
//  x+=50; drawFixedLbl(objPrefix+dir+".profit"+n,profit, 0, x,y, 10, "Courier", TextColor, true);

  //Print(dirs[dir]+":"+n+":"+nb);
}

//--------------------------------------------------------------------------------------
void drawFixedLbl(string objname, string s, int Corner, int DX, int DY, int FSize, string Font, color c, bool bg)
//--------------------------------------------------------------------------------------
{
   if (ObjectFind(objname) < 0) ObjectCreate(objname, OBJ_LABEL, 0, 0, 0);
   
   ObjectSet(objname, OBJPROP_CORNER, Corner);
   ObjectSet(objname, OBJPROP_XDISTANCE, DX);
   ObjectSet(objname, OBJPROP_YDISTANCE, DY);
   ObjectSet(objname,OBJPROP_BACK, bg);      
   ObjectSetText(objname, s, FSize, Font, c);
} //drawFixedLbl

//--------------------------------------------------------------------------------------
void RemoveObjects(string Pref)
//--------------------------------------------------------------------------------------
{   
   int i;
   string objname = "";

   for (i = ObjectsTotal(); i >= 0; i--) {
      objname = ObjectName(i);
      if (StringFind(objname, Pref, 0) > -1) ObjectDelete(objname);
   }
} /* RemoveObjects*/

//end

